pro reconstruct_aver_drift, drifts_save, npairs_save, file_scalars=file_scalars, $
                            ind_scans=ind_scans, ind_subscans=ind_subscans, $
                            noise_level=noise_level, time_1=time_1, t_proc=t_proc


;;; computes absolute average drift as a function of time, from drift differences matrix
;;; solution not unique, containing an oscillatory component removed in subtract_aver_drift.pro

;;;;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;;; copyright 2007-2011 H\'el\`ene Roussel
;;; roussel@iap.fr
;;; Institut d'Astrophysique de Paris - Universit\'e Pierre et Marie Curie, France

;;; download and documentation: http://www2.iap.fr/users/roussel/herschel/

;;; Scanamorphos is distributed under the terms of the GNU General Public License
;;; (http://www.gnu.org/licenses/gpl.html).

;;; July 2010: public release version 1

;;;;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



restore, file_scalars
epsilon = (Machar()).eps

restore, file_diff_drifts


if keyword_set(visu) then begin
   time_2 = systime(/seconds)
   t_proc += (time_2 - time_1)
   if nt_save gt 1.e+4 then print, 'drift differences matrix too large to be displayed' $
      else disp_ima, win=5, diff_drifts, min_map=-noise_level/3., max_map=noise_level/3., $
      title='matrix of average drift differences', col_tab=15
   time_1 = systime(/seconds)
endif



drifts_save = fltarr(nt_save)
npairs_save = fltarr(nt_save)


n_tj = fltarr(nt_save)
w = where(diff_drifts(1:*, 0) gt 0., c2)
n_tj(0) = c2
for it1 = 1L, nt_save - 2L do begin
   w = where([reform(diff_drifts(it1, 0:it1-1)), diff_drifts(it1+1:*, it1)] gt 0., c2)
   n_tj(it1) = c2
endfor
w = where(diff_drifts(nt_save-1, 0:nt_save-2) gt 0., c2)
n_tj(nt_save-1) = c2

ind_tj_sort = reverse(sort(n_tj))

it_start = ind_tj_sort(0)
drifts_save(it_start+1:*) = -diff_drifts(it_start, it_start+1:*)
npairs_save(it_start+1:*) = diff_drifts(it_start+1:*, it_start)
drifts_save(0:it_start-1) = diff_drifts(0:it_start-1, it_start)
npairs_save(0:it_start-1) = diff_drifts(it_start, 0:it_start-1)
cw_min = max([floor(0.25 * n_tj(ind_tj_sort(1))), 3])


iter_rec = 0

repeat begin
   w = where(npairs_save gt 0., ntot_prev)
   drifts_save_prev = drifts_save
   npairs_save_prev = npairs_save

   w = where(n_tj(ind_tj_sort) ge cw_min, cd)
   ind_tj = ind_tj_sort(w)
   nperm = 0L

   if iter_rec eq 0 then k = 1L else k = 0L
   while (k lt cd and nperm le cd - k) do begin
      it = ind_tj(k)
      if (it gt 0 and it lt nt_save - 1) then begin
         drifts = [diff_drifts(0:it-1, it), 0., -reform(diff_drifts(it, it+1:*))]
         npairs = [reform(diff_drifts(it, 0:it-1)), 0., diff_drifts(it+1:*, it)]
      endif else if it eq 0 then begin
         drifts = [0., -reform(diff_drifts(it, it+1:*))]
         npairs = [0., diff_drifts(it+1:*, it)]
      endif else begin
         drifts = [diff_drifts(0:it-1, it), 0.]
         npairs = [reform(diff_drifts(it, 0:it-1)), 0.]
      endelse
      mult_npairs = npairs * npairs_save
      w = where(mult_npairs gt 0., cw)
      if cw ge cw_min then begin
         off = total((drifts - drifts_save) * mult_npairs) / total(mult_npairs)
         sum_npairs = npairs + npairs_save
         drifts_save = ((drifts - off) * npairs + drifts_save * npairs_save) $
            / (sum_npairs + (sum_npairs lt epsilon))
         npairs_save = sum_npairs
         nperm = 0L
         k += 1L
      endif else begin
         ind_tj(k:cd-1) = shift(ind_tj(k:cd-1), -1)
         nperm += 1L
      endelse
   endwhile

   iter_rec += 1
   if iter_rec eq 1 then cw_min = max([cw_min / 2, 3]) else cw_min = 3

   w = where(npairs_save gt 0., ntot)
   resi = max(abs(drifts_save_prev(w) - drifts_save(w)))
   if keyword_set(debug) then print, 'iter ' + chain(iter_rec) + ': resi / noise_level = ' + chain(resi / noise_level, 2)
endrep until ((resi lt 0.1 * noise_level and ntot le ntot_prev and iter_rec ge 3) or iter_rec ge 12)


end
